/*
 * Project name:
     RTOS example - Example 2
 * Author:
     Stevan Tosic
 * Date:
     06.10.2011.
 * Revision History:
       - initial release;
 * Description:
     Two computing tasks (0,4), two semaphore driven tasks (1,2)
     and one timer (1sec) task (3).
 * Test configuration:
     MCU:             P18F8520
     dev.board:       bigpic5
     Oscillator:      HS, 8.0 MHz
     Ext. Modules:    -
     SW:              mikroC PRO for PIC 4.60.00.
 * NOTES:
     - RTOS semaphores
*/

#include "rtos.h"
unsigned char TaskStack[NTASKS][TASK_STACK];
SEM s1, s2;
unsigned i0=0, i4=0;
double si0, si4;

void Task0(void)
{
       while(1){
             for(i0=0; i0<50;i0++) {
                 si0 = sin(1.0)*3.0 * 4.0/ 2.0;
             }
             PORTB.F0=~PORTB.F0;
        }
}

void Task1(void)
{
   while(1){
         RTOS_Sleep(500);
         RTOS_OpenSemaphore(&s1);
         PORTB.F1=~PORTB.F1;
  }
}

void Task2(void)
{
   int i;
   while(1){
         RTOS_Sleep(500);
         RTOS_OpenSemaphore(&s2);
         PORTB.F2=~PORTB.F2;
     }
}

void Task3(void)
{
   while(1){
         RTOS_Sleep(1000);
         PORTB.F3=~PORTB.F3;
  }
}

void Task4(void)
{
       while(1){
             for(i4=0; i4<50;i4++) {
                 si4 = sin(1.0)*3.0 * 4.0/ 2.0;
             }
             PORTB.F4=~PORTB.F4;
        }
}

void main() {
  CMCON |= 0x07;         // turn off comparators
  ADCON1 |= 0x0C;
  TRISB = 0;
  PORTB = 0x00;
  Delay_ms(1000);

  RTOS_CreateSemaphore(&s1, 10);
  RTOS_CreateSemaphore(&s2, 6);

  RTOS_Init();
  RTOS_CreateTask((TaskType)Task0,TaskStack[0], sizeof (TaskStack[0]), 0);
  RTOS_CreateTask((TaskType)Task1,TaskStack[1], sizeof (TaskStack[1]), 1);
  RTOS_CreateTask((TaskType)Task2,TaskStack[2], sizeof (TaskStack[2]), 2);
  RTOS_CreateTask((TaskType)Task3,TaskStack[3], sizeof (TaskStack[3]), 3);
  RTOS_CreateTask((TaskType)Task4,TaskStack[4], sizeof (TaskStack[4]), 4);
  RTOS_StartScheduler();
  while(1){}
}