/*
 * Project name:
     RTOS example - Example 3
 * Author:
     Stevan Tosic
 * Date:
     06.10.2011.
 * Revision History:
       - initial release;
 * Description:
     Two tasks (1,3) waiting tasks signal (3,4) on semaphores and one
     idle task (0).
 * Test configuration:
     MCU:             P18F8520
     dev.board:       bigpic5
     Oscillator:      HS, 8.0 MHz
     Ext. Modules:    -
     SW:              mikroC PRO for PIC 4.60.00.
 * NOTES:
     - RTOS semaphores
*/
#include "rtos.h"
unsigned char TaskStack[NTASKS][TASK_STACK];
SEM s0, s1, s2, s3;

void Task0(void)
{
        while(1){
            RTOS_Yield();
        }
}

void Task1(void)
{
   while(1){
         RTOS_OpenSemaphore(&s0);
         PORTB.F1=~PORTB.F1;
  }
}

void Task2(void)
{
   while(1){
         RTOS_Sleep(1000);
         PORTB.F2=~PORTB.F2;
         RTOS_ReleaseSemaphore(&s0);
     }
}

void Task3(void)
{
   while(1){
         RTOS_OpenSemaphore(&s1);
         PORTB.F3=~PORTB.F3;
  }
}

void Task4(void)
{
   while(1){
         RTOS_Sleep(4000);
         PORTB.F4=~PORTB.F4;
         RTOS_ReleaseSemaphore(&s1);
     }
}

void main() {
  CMCON |= 0x07;         // turn off comparators
  ADCON1 |= 0x0C;
  TRISB = 0;
  PORTB = 0x00;
  Delay_ms(500);

  RTOS_CreateSemaphore(&s0, 0);
  RTOS_CreateSemaphore(&s1, 0);
  RTOS_CreateSemaphore(&s2, 0);
  RTOS_CreateSemaphore(&s3, 0);

  RTOS_Init();
  RTOS_CreateTask((TaskType)Task0,TaskStack[0], sizeof (TaskStack[0]), 0);
  RTOS_CreateTask((TaskType)Task1,TaskStack[1], sizeof (TaskStack[1]), 1);
  RTOS_CreateTask((TaskType)Task2,TaskStack[2], sizeof (TaskStack[2]), 2);
  RTOS_CreateTask((TaskType)Task3,TaskStack[3], sizeof (TaskStack[3]), 3);
  RTOS_CreateTask((TaskType)Task4,TaskStack[4], sizeof (TaskStack[4]), 4);
  RTOS_StartScheduler();
  while(1){}
}